/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gestures for Mozilla.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

mgStorage = {
  RDF : Components.classes["@mozilla.org/rdf/rdf-service;1"]
        .getService(Components.interfaces.nsIRDFService),

  RDFCUtils : Components.classes["@mozilla.org/rdf/container-utils;1"]
              .getService()
              .QueryInterface(Components.interfaces.nsIRDFContainerUtils),

  NS : "http://optimoz.mozdev.org/gestures-rdf#",

  userDS : null,
  defaultMappingsDS : null,

  getUserRDF : function() {
    if (this.userDS) {
      return this.userDS;
    } else {
      var mgUserFile = this.getProfileDir();
      mgUserFile.append("mousegestures.rdf");
      this.userDS = this.RDF.GetDataSourceBlocking(this.fileToURL(mgUserFile).spec);
      return this.userDS;
    }
  },

  getDefaultMappings : function() {
    if (!this.defaultMappingsDS) {
      this.defaultMappingsDS = this.RDF
          .GetDataSourceBlocking("chrome://mozgest/content/defaultMappings.rdf");
    }
    return this.defaultMappingsDS;
  },

  initUserRDF : function() {
    var dsUser = this.getUserRDF();
  
    var usInfo = this.RDF.GetResource("urn:mozgest:userstore");
    // determine which mozgest-version saved this userstore. default to 0 if 
    //    userstore doesnt exist - this will trigger installation of mappings below
    var userstoreVersion = this.getProperty(dsUser, usInfo, "creatorVersion") || 0;
  
    var dsDefault = this.getDefaultMappings();
    var releaseInfo = this.RDF.GetResource("urn:mozgest:release");
    var releaseVersion = this.getProperty(dsDefault, releaseInfo, "version");
  
    if (releaseVersion > userstoreVersion) {
      this.unassertLiteral(dsUser, usInfo, "creatorVersion", userstoreVersion);
      this.assertLiteral(dsUser, usInfo, "creatorVersion", releaseVersion);
      // enumerate all resources from defaultMappings file
      var elements = dsDefault.GetAllResources();
      while (elements.hasMoreElements()) {
        var el = elements.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
        // only process Seqs
        if(this.RDFCUtils.IsSeq(dsDefault, el)) {
  //        var seqRes = this.RDF.GetResource(el.Value);
          var seqRes = el;
          var sourceSeq = this.newContainer();
          sourceSeq.Init(dsDefault, el);
          var targetSeq = this.newContainer();
          // init current seq in userstore, or create it if it doesn't exist
          var emptySeq = false;
          try {
            targetSeq.Init(dsUser, seqRes);
          } catch (e) {
            this.RDFCUtils.MakeSeq(dsUser, seqRes);
            targetSeq.Init(dsUser, seqRes);
            emptySeq = true;
          }
          var mappingsEnum = sourceSeq.GetElements();
          while (mappingsEnum.hasMoreElements()) {
            // get mapping data
            var mapping = mappingsEnum.getNext();
  
            // determine which mozgest version introduced this mapping.
            //    default to 0.4 if no "since" property present
            var mSince = this.getProperty(dsDefault, mapping, "since") || 0.4;
  
            // if mapping is newer than userstoreVersion, install it
            if (mSince > userstoreVersion) {
              if (emptySeq ||
                  !this.findMapping(this.getProperty(dsDefault, mapping, "code"),
                                    targetSeq)) {
                this.installMapping(mapping, targetSeq);
              }
            }
          }
        }
      }
      dsUser.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
    }
    dsUser.AddObserver(mgMappingsObserver);
  },

  // lookup mapping by code
  findMapping : function(code, targetContainer) {
    var dsUser = this.getUserRDF();
    var mappingsEnum = targetContainer.GetElements();
    while (mappingsEnum.hasMoreElements()) {
      var mapping = mappingsEnum.getNext();
      if (this.getProperty(dsUser, mapping, "code") == code) {
        return mapping;
      }
    }
    return false;
  },

  // installs a mapping from defaultMappings to userstore.
  installMapping : function(mapping, targetContainer) {
    var dsUser = this.getUserRDF();
    var dsDefault = this.getDefaultMappings();
  
    var mCode = this.getProperty(dsDefault, mapping, "code");
    var mFunction = this.getProperty(dsDefault, mapping, "function");
    var mAppearance = this.getProperty(dsDefault, mapping, "appearance");
  
    node = this.RDF.GetAnonymousResource();
    this.assertLiteral(dsUser, node, "code", mCode);
    this.assertLiteral(dsUser, node, "function", mFunction);
    if (mAppearance) {
      this.assertLiteral(dsUser, node, "appearance", mAppearance);
    }
    targetContainer.AppendElement(node, true);
  },

  readRDFGestureTable : function(component) {
    // get our datasource
    var dsource = this.getUserRDF();
  
    // instantiate a container object and its root node
    var mappingsContainer = this.newContainer();
    var rootnode = this.RDF.GetResource("urn:mozgest:mappings:" + component);
  
    try {
      mappingsContainer.Init(dsource, rootnode);
    } catch (e) {
      // no existing mappings
      return;
    }
  
    var enumerator = mappingsContainer.GetElements();
    while (enumerator.hasMoreElements()) {
      var mapping = enumerator.getNext()
                    .QueryInterface(Components.interfaces.nsIRDFResource);
      var entry = new Mapping();
      entry.resource = mapping.Value;
      entry.code = this.getProperty(dsource, mapping, "code");
      entry.appearance = this.getProperty(dsource, mapping, "appearance");
      entry.func = this.getProperty(dsource, mapping, "function");
      entry.custom = decodeURI(this.getProperty(dsource, mapping, "custom"));
      entry.name = (entry.func) ? bundle.getString(entry.func)
                                : this.getProperty(dsource, mapping, "name");
      entry.countBase = Math.abs(this.getProperty(dsource, mapping, "count")) || 0;
      addGesture(entry);
    }
  },

  saveGestureCounts : function(e) {
    // do not react on html page-loads (they fire "unloads" as well)
    if (e.view == window) {
      store = window.mgStorage;
      dump("MozGest: Saving gesture counts\n");
      var dsource = store.getUserRDF();
      for (var code in gesturesTable) {
        mapping = gesturesTable[code];
        if (mapping.countCur != 0) {
          // get currently saved value
          var node = store.RDF.GetResource(mapping.resource);
          var old = Math.abs(store.getProperty(dsource, node, "count")) || 0;
    
          // add current session count and save it
          var n = old + mapping.countCur;
          store.changeLiteral(dsource, node, "count", old, n);
        }
      }
    }
  },


  /*** convenience functions for RDF access ***/

  newContainer : function() {
    return Components.classes['@mozilla.org/rdf/container;1'].createInstance()
           .QueryInterface(Components.interfaces.nsIRDFContainer);
  },

  assertLiteral : function(dsource, node, propName, value) {
    dsource.Assert(node, this.RDF.GetResource(this.NS + propName), 
                   this.RDF.GetLiteral(value), true);
  },

  unassertLiteral : function(dsource, node, propName, value) {
    dsource.Unassert(node, this.RDF.GetResource(this.NS + propName), 
                   this.RDF.GetLiteral(value), true);
  },

  changeLiteral : function(dsource, node, propName, oVal, nVal) {
    dsource.Change(node, this.RDF.GetResource(this.NS + propName), 
                   this.RDF.GetLiteral(oVal), this.RDF.GetLiteral(nVal));
  },

  getProperty : function (dsource, obj, propName) {
    // get nsIRDFNode
    var literal = dsource.GetTarget(obj,
                                    this.RDF.GetResource(this.NS + propName),
                                    true);
    if (literal) {
      // convert to nsIRDFLiteral and get the value
      return literal.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    } else {
      return null;
    }
  },


  /*** profile & URL handling ***/

  getProfileDir : function() {
    // get a File representing user's profile dir
    var profileInternal = Components.classes["@mozilla.org/profile/manager;1"]
        .createInstance().QueryInterface(Components.interfaces.nsIProfileInternal);
    return profileInternal.getProfileDir(profileInternal.currentProfile);
  },


  fileToURL : function(aFile) {
    var fileHandler = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);
    return fileHandler.newFileURI(aFile);
  },

}



/*** observer stuff ***/

var mgMappingsObserver = new Object();
mgMappingsObserver = {
  updateTimer : null,

  // throughout this observer, we need the prefix "window." to access
  // mozgest's global variables - it seems we're somehow "outside" of the window

  delayedUpdate : function() {
    window.clearTimeout(this.updateTimer);
    this.updateTimer = window.setTimeout(loadGestureTable, 500);
  },

  onAssert: function(aDataSource, aSource, aProperty, aTarget) {
    if (aProperty.Value.indexOf(this.NS) == 0) {
      this.delayedUpdate();
    }
  },

  onUnassert : function(aDataSource, aSource, aProperty, aTarget) {
    if (aProperty.Value.indexOf(this.NS) == 0) {
      this.delayedUpdate();
    }
  },

  onChange: function(aDataSource, aSource, aProperty, aTarget, aOldTarget,
                     aNewTarget) {
    if (aProperty.Value.indexOf(window.mgStorage.NS) == 0
        && aProperty.Value != window.mgStorage.NS + "count") {
      this.delayedUpdate();
    }
  },

  onMove: function(aDataSource, aOldSource, aNewSource, aProperty,
  aTarget) {
  },

  beginUpdateBatch: function(aDataSource) {
  },

  endUpdateBatch: function(aDataSource) {
  },

}
